@@ -8,16 +8,19 @@ smaller and more feature-oriented suite can be run with::
     python -m unittest discover jplephem
 
 """
+import gc
 import mmap
 import numpy as np
 import sys
 import tempfile
+import warnings
 from doctest import DocTestSuite, ELLIPSIS
 from functools import partial
 from io import BytesIO
 from jplephem import Ephemeris, commandline
 from jplephem.exceptions import OutOfRangeError
 from jplephem.daf import DAF, FTPSTR, NAIF_DAF
+from jplephem.pck import PCK
 from jplephem.spk import SPK
 from struct import Struct
 try:
@@ -44,11 +47,10 @@ target_names = {
     'mars': 499,           # w.r.t. 4 MARS BARYCENTER
     }
 
-
 class TestDAFBytesIO(TestCase):
     def sample_daf(self):
-        word = Struct('d').pack
-        integer = Struct('i').pack
+        word = Struct('<d').pack
+        integer = Struct('<i').pack
         return BytesIO(b''.join([
             # Record 1 - File Record
             b'DAF/SPK ',
@@ -156,13 +158,13 @@ class TestDAFBytesIO(TestCase):
 
         # Update n_summaries of final summary block to full.
         d.file.seek(6 * 1024 + 16)
-        d.file.write(Struct('d').pack(d.summaries_per_record))
+        d.file.write(Struct('<d').pack(d.summaries_per_record))
 
         d.add_array(b'Summary Name 3', (121.0, 232.0, 343), [3003.0] * 200)
 
         # Reset n_summaries of that block back to its real value.
         d.file.seek(6 * 1024 + 16)
-        d.file.write(Struct('d').pack(1))
+        d.file.write(Struct('<d').pack(1))
 
         summaries = list(d.summaries())
         eq = self.assertEqual
@@ -176,7 +178,6 @@ class TestDAFBytesIO(TestCase):
         eq(list(d.map(summaries[1][1])), [2002.0] * 128)
         eq(list(d.map(summaries[2][1])), [3003.0] * 200)
 
-
 class TestDAFRealFile(TestDAFBytesIO):
     # Where "Real" = "written to disk with a real file descriptor
     # instead of an in-memory BytesIO".
@@ -188,7 +189,6 @@ class TestDAFRealFile(TestDAFBytesIO):
         f.seek(0)
         return f
 
-
 def fake_mmap_that_raises_OSError(*args, **kw):
     raise OSError('mmap() not supported on this platform')
 
@@ -204,7 +204,6 @@ class TestDAFRealFileWithoutMMap(TestDAFRealFile):
     def tearDown(self):
         mmap.mmap = self.mmap
 
-
 class _CommonTests(object):
 
     def check0(self, xyz, xyzdot=None):
@@ -313,7 +312,6 @@ class _CommonTests(object):
         tdb = self.jomega + 16.01
         self.assertRaises(ValueError, self.position, 'earthmoon', tdb)
 
-
 class SPKTests(_CommonTests, TestCase):
 
     def setUp(self):
@@ -328,6 +326,17 @@ class SPKTests(_CommonTests, TestCase):
     def tearDown(self):
         self.spk.close()
 
+        if sys.version_info < (3,):
+            return
+
+        # With thanks for https://stackoverflow.com/questions/24717027/
+        with warnings.catch_warnings(record=True) as w:
+            warnings.resetwarnings()
+            warnings.simplefilter('always', ResourceWarning)
+            del self.spk
+            gc.collect()
+            self.assertFalse(w and str(w[-1]))
+
     def position(self, name, tdb, tdb2=0.0):
         segment = self.spk[0, target_names[name]]
         return segment.compute(tdb, tdb2)
@@ -347,9 +356,9 @@ class SPKTests(_CommonTests, TestCase):
         segment = self.spk[0,4]
         self.assertEqual(str(segment), segment.describe(verbose=False))
         self.assertEqual(segment.describe(verbose=False),
-  '2414864.50..2471184.50  Type 2  Solar System Barycenter (0) -> Mars Barycenter (4)')
+  '1899-07-29..2053-10-09  Type 2  Solar System Barycenter (0) -> Mars Barycenter (4)')
         self.assertEqual(segment.describe(verbose=True),
-  '2414864.50..2471184.50  Type 2  Solar System Barycenter (0) -> Mars Barycenter (4)'
+  '1899-07-29..2053-10-09  Type 2  Solar System Barycenter (0) -> Mars Barycenter (4)'
   '\n  frame=1 source=DE-0421LE-0421')
 
     def test_loading_array(self):
@@ -364,10 +373,17 @@ class SPKTests(_CommonTests, TestCase):
             segment.compute_and_differentiate(tdb)
         except OutOfRangeError as e:
             self.assertEqual(str(e), 'segment only covers dates'
-                             ' 2414864.5 through 2471184.5')
+                             ' 1899-07-29 through 2053-10-09')
             self.assertIs(type(e.out_of_range_times), np.ndarray)
             self.assertEqual(list(e.out_of_range_times), [True, False, True])
 
+    def test_whether_bad_ephemeris_leaves_file_open(self):
+        # This doesn't actually fail if the file object is left open,
+        # but should produce a ResourceWarning as a side effect.
+        non_spk_path = __file__
+        with self.assertRaises(ValueError):
+            SPK.open(non_spk_path)
+
 class LegacyTests(_CommonTests, TestCase):
 
     def setUp(self):
@@ -406,6 +422,14 @@ class LegacyTests(_CommonTests, TestCase):
         self.assertAlmostEqual(y, 1.05103857e+08, delta=1.0)
         self.assertAlmostEqual(z, 45550861.44383482, delta=epsilon_m)
 
+class PCKTests(TestCase):
+    def test_out_of_range_date(self):
+        p = PCK.open('moon_pa_de421_1900-2050.bpc')
+        segment = p.segments[0]
+        expect = 'segment only covers dates 1900-01-01 through 2051-01-01'
+        with self.assertRaisesRegex(ValueError, expect):
+            segment.compute(0.0, 0.0)
+        p.close()
 
 class NAIF_DAF_Tests(TestCase):
 
@@ -417,7 +441,6 @@ class NAIF_DAF_Tests(TestCase):
             self.assertAlmostEqual(y, 4.25141646e+07, delta=2.0)
             self.assertAlmostEqual(z, 1.39379183e+07, delta=2.0)
 
-
 class CommandLineTests(TestCase):
     maxDiff = 9999
 
@@ -448,23 +471,142 @@ class CommandLineTests(TestCase):
     def test_spk_command(self):
         self.assertEqual(commandline.main(['spk', 'de405.bsp']), """\
 File type NAIF/DAF and format BIG-IEEE with 15 segments:
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Mercury Barycenter (1)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Venus Barycenter (2)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Earth Barycenter (3)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Mars Barycenter (4)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Jupiter Barycenter (5)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Saturn Barycenter (6)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Uranus Barycenter (7)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Neptune Barycenter (8)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Pluto Barycenter (9)
-2433282.50..2469807.50  Type 2  Solar System Barycenter (0) -> Sun (10)
-2433282.50..2469807.50  Type 2  Earth Barycenter (3) -> Moon (301)
-2433282.50..2469807.50  Type 2  Earth Barycenter (3) -> Earth (399)
-2433282.50..2469807.50  Type 2  Mercury Barycenter (1) -> Mercury (199)
-2433282.50..2469807.50  Type 2  Venus Barycenter (2) -> Venus (299)
-2433282.50..2469807.50  Type 2  Mars Barycenter (4) -> Mars (499)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Mercury Barycenter (1)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Venus Barycenter (2)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Earth Barycenter (3)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Mars Barycenter (4)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Jupiter Barycenter (5)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Saturn Barycenter (6)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Uranus Barycenter (7)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Neptune Barycenter (8)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Pluto Barycenter (9)
+1950-01-01..2050-01-01  Type 2  Solar System Barycenter (0) -> Sun (10)
+1950-01-01..2050-01-01  Type 2  Earth Barycenter (3) -> Moon (301)
+1950-01-01..2050-01-01  Type 2  Earth Barycenter (3) -> Earth (399)
+1950-01-01..2050-01-01  Type 2  Mercury Barycenter (1) -> Mercury (199)
+1950-01-01..2050-01-01  Type 2  Venus Barycenter (2) -> Venus (299)
+1950-01-01..2050-01-01  Type 2  Mars Barycenter (4) -> Mars (499)
 """)
 
+    def test_verbose_spk_command_with_tidy_ephemeris(self):
+        output = commandline.main(['spk', '-v', 'de421.bsp'])
+        pieces = output.split('\n\n')
+        pieces[1:-2] = ['...']
+        output = '\n\n'.join(pieces)
+        self.assertEqual(output, """\
+File type DAF/SPK and format LTL-IEEE with 15 segments:
+1899-07-29..2053-10-09  Type 2  Solar System Barycenter (0) -> Mercury Barycenter (1)
+   7040 polynomials covering 8.0 days each
+      x 14 coefficients per polynomial
+      x 3 coordinates
+      = 295680 double precision floats
+   Polynomial start date matches segment start date
+   Polynomial end date matches segment end date
+
+...
+
+1899-07-29..2053-10-09  Type 2  Mars Barycenter (4) -> Mars (499)
+   1 polynomial covering 56320.0 days
+      x 2 coefficients per polynomial
+      x 3 coordinates
+      = 6 double precision floats
+   Polynomial start date matches segment start date
+   Polynomial end date matches segment end date
+
+""")
+
+    def test_verbose_spk_command_with_untidy_ephemeris(self):
+        output = commandline.main(['spk', '-v', 'de442s.bsp'])
+        pieces = output.split('\n\n')
+        pieces[1:-2] = ['...']
+        output = '\n\n'.join(pieces)
+        self.assertEqual(output, """\
+File type DAF/SPK and format LTL-IEEE with 14 segments:
+1849-12-26..2150-01-22  Type 2  Venus Barycenter (2) -> Venus (299)
+   1 polynomial covering 401792.0 days
+      x 2 coefficients per polynomial
+      x 3 coordinates
+      = 6 double precision floats
+   First polynomial starts 109568.0 days earlier than segment start date, on 1549-12-31
+   Final polynomial ends 182624.0 days later than segment end date, on 2650-01-25
+
+...
+
+1849-12-26..2150-01-22  Type 2  Solar System Barycenter (0) -> Mercury Barycenter (1)
+   13701 polynomials covering 8.0 days each
+      x 14 coefficients per polynomial
+      x 3 coordinates
+      = 575442 double precision floats
+   Polynomial start date matches segment start date
+   Final polynomial ends 8.0 days later than segment end date, on 2150-01-30
+
+""")
+
+    def test_excerpt_command(self):
+        output = commandline.main(['excerpt', '2023/8/23', '2023/8/24',
+                                   'de421.bsp', 'de421_excerpt.bsp'])
+        self.assertEqual(output, """\
+Date 2023/8/23  = JD 2460179.5
+Date 2023/8/24  = JD 2460180.5
+
+'de421_excerpt.bsp' written successfully with the following contents
+
+File type DAF/SPK and format LTL-IEEE with 15 segments:
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Mercury Barycenter (1)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Venus Barycenter (2)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Earth Barycenter (3)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Mars Barycenter (4)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Jupiter Barycenter (5)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Saturn Barycenter (6)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Uranus Barycenter (7)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Neptune Barycenter (8)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Pluto Barycenter (9)
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Sun (10)
+2023-08-23..2023-08-24  Type 2  Earth Barycenter (3) -> Moon (301)
+2023-08-23..2023-08-24  Type 2  Earth Barycenter (3) -> Earth (399)
+2023-08-23..2023-08-24  Type 2  Mercury Barycenter (1) -> Mercury (199)
+2023-08-23..2023-08-24  Type 2  Venus Barycenter (2) -> Venus (299)
+2023-08-23..2023-08-24  Type 2  Mars Barycenter (4) -> Mars (499)
+""")
+
+        preface = """\
+;
+; This is an ephemeris excerpt created by jplephem 2.23, which was
+; asked to narrow the ephemeris to Julian dates 2460179.5 - 2460180.5
+; (proleptic Gregorian dates 2023-08-23 through 2023-08-24).
+;
+; Here is the comments area from the original ephemeris file:
+; ----------------------------------------------------------------------
+"""
+        output1 = commandline.main(['comment', 'de421.bsp'])
+        output2 = commandline.main(['comment', 'de421_excerpt.bsp'])
+        self.assertEqual(preface + output1, output2)
+
+        output = commandline.main(['spk', '-v', 'de421_excerpt.bsp'])
+        pieces = output.split('\n\n')
+        pieces[1:-2] = ['...']
+        output = '\n\n'.join(pieces)
+        self.assertEqual(output, """\
+File type DAF/SPK and format LTL-IEEE with 15 segments:
+2023-08-23..2023-08-24  Type 2  Solar System Barycenter (0) -> Mercury Barycenter (1)
+   1 polynomial covering 8.0 days
+      x 14 coefficients per polynomial
+      x 3 coordinates
+      = 42 double precision floats
+   First polynomial starts 3.0 days earlier than segment start date, on 2023-08-20
+   Final polynomial ends 4.0 days later than segment end date, on 2023-08-28
+
+...
+
+2023-08-23..2023-08-24  Type 2  Mars Barycenter (4) -> Mars (499)
+   1 polynomial covering 56320.0 days
+      x 2 coefficients per polynomial
+      x 3 coordinates
+      = 6 double precision floats
+   First polynomial starts 45315.0 days earlier than segment start date, on 1899-07-29
+   Final polynomial ends 11004.0 days later than segment end date, on 2053-10-09
+
+""")
 
 def load_tests(loader, tests, ignore):
     """Run our main documentation as a test."""
